/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.anweisung;

import java.util.Collection;
import java.util.List;

import de.bsvrz.ibv.uda.interpreter.AnweisungsBlock;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.Fehler;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext;
import de.bsvrz.sys.funclib.bitctrl.interpreter.logik.LogischerWert;

/**
 * die Repräsentation eines bedingten Anweisungsblocks.<br>
 * Bedingte Anweisungsblöcke werden innerhalb einer WennAnweisung verwendet, um
 * die einzelnen Alternativen bei der Ausführung der Anweisung darzustellen.<br>
 * Die Auswertung des Ausdrucks entspricht der Auswertung eines
 * Anweisungsblocks. Im Gegensatz zum "einfachen" Anweisungsblock werden die
 * Anweisungen jedoch nur ausgeführt, wenn die enthaltene Bedingung gültig ist.
 * <br>
 * Das Ergebnis der Auswertung eines "Bedingten Anweisungsblocks" ist ein
 * logischer Wert, der beschreibt, ob der Anweisungsblock ausgeführt wurde oder
 * nicht.<br>
 * Eine bei der Ausführung des Anweisungsblocks nicht abgefangene Ausnahme wird
 * nach außen weitergereicht.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class BedingterAnweisungsBlock extends AnweisungsBlock {

	/**
	 * die Ausdruck, der die Bedingung für die Auswertung beschreibt.
	 */
	private final Ausdruck bedingung;

	/**
	 * Konstruktor.
	 *
	 * @param zeile
	 *            die Nummer der Zeile, in der der Anweisungsblock im Quelltext
	 *            definiert ist
	 * @param skript
	 *            der Name des Skripts
	 * @param bedingung
	 *            die Bedingung für die Ausführung des Anweisungen
	 * @param anweisungen
	 *            die auszuführenden Anweisungen
	 */
	public BedingterAnweisungsBlock(final int zeile, final String skript, final Ausdruck bedingung,
			final Collection<Anweisung> anweisungen) {
		super(zeile, skript);
		this.bedingung = bedingung;
		addAnweisungen(anweisungen);
	}

	/**
	 * ermittelt, ob die Bedingung düe die Ausführung des Blocks erfüllt ist.
	 *
	 * @param kontext
	 *            der Konetxt für die Auswerting der Bedingung
	 * @return wahr, wenn die Bedingung gültig ist
	 */
	public boolean isGueltig(final Kontext kontext) {
		try {

			return ((LogischerWert) bedingung.interpret(kontext)).get();
		} catch (@SuppressWarnings("unused") final ClassCastException e) {
			throw new Fehler("Bedingung liefert kein boolsches Ergebnis");
		}
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.AnweisungsBlock#getNachfolger()
	 */
	@Override
	public List<Ausdruck> getNachfolger() {
		final List<Ausdruck> liste = super.getNachfolger();
		liste.add(0, bedingung);
		return liste;
	}
}
