/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.anweisung;

import de.bsvrz.ibv.uda.interpreter.ausdruck.Struktur;
import de.bsvrz.ibv.uda.interpreter.daten.SkriptKontext;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ObjektZugriffsFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehlerSubtyp;
import de.bsvrz.sys.funclib.bitctrl.interpreter.NichtWert;
import de.bsvrz.sys.funclib.bitctrl.interpreter.logik.LogischerWert;

/**
 * abstrakte Implementierung einer Aktivität bei der Ausführung eines
 * UdaSkripts.<br>
 * Eine Aktivität ist eine nebenläufige Aktion die parallel zum Hauptskript
 * ausgeführt werden kann. Folgende nebenläufige Aktivitäten sind in Uda
 * möglich:
 * <ul>
 * <li><b><i>Zyklus</i></b>: eine Operation die in vorgegebenen Zeitintervallen
 * ausgeführt wird.</li>
 * <li><b><i>Anmeldung</i></b>: eine Struktur, die über eine
 * Datenverteilerverbindung asynchron mit Daten vom Datenverteiler versorgt wird
 * und diese Daten weiterverarbeitet.</li>
 * </ul>
 * Die Klasse implementiert die Schnittstelle Struktur, d.h. auf bestimmte
 * Attribute kann mit dem Elementzugriffoperator zugegriffen werden. Folgende
 * Attribute werden unterstützt:
 * <ul>
 * <li><b>aktiv</b>: liefert den Zustand der Aktivität als Wahrheitswert</li>
 * <li><b>abgebrochen</b>: liefert den Fehlerzustand der Aktivität als
 * Wahrheitswert</li>
 * <li><b>fehler</b>: liefert das Fehlerobjekt der Aktivität, dass beim Abbruch
 * gesetzt wird oder den Wert "undefiniert"</li>
 * </ul>
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public abstract class UdaAktivitaet implements Struktur {

	/**
	 * der lokale Kontext der Aktivität.
	 */
	private final SkriptKontext kontext;

	/**
	 * der Marker zur Definition des Aktivitätszustands.
	 */
	private boolean running;

	/**
	 * das Fehlerobjekt, das einen Wert erhält, wenn die Aktivität unerwartet
	 * beendet wurde.
	 */
	private UdaFehler fehler;

	/**
	 * Konstruktor.
	 *
	 * @param kontext
	 *            der umgebende Kontext der Anweisung
	 * @param name
	 *            der name unter dem die Aktivität im umgebenden Kontext
	 *            registriert wird
	 */
	public UdaAktivitaet(final SkriptKontext kontext, final String name) {
		this.kontext = new SkriptKontext(kontext, false);
		if ((name != null) && (name.length() > 0)) {
			kontext.set(name, this);
			this.kontext.set(name, this);
		}
	}

	/**
	 * beendet die Aktivität.
	 */
	public abstract void beenden();

	/**
	 * liefert den Status der Aktivität (läuft/läuft nicht mehr).
	 *
	 * @return den Status
	 */
	public boolean isRunning() {
		return running;
	}

	/**
	 * setzt den Status der Ausführung der Aktivität.
	 *
	 * @param running
	 *            der Wert für den Status
	 */
	protected void setRunning(final boolean running) {
		this.running = running;
	}

	/**
	 * {@inheritDoc}.<br>
	 * Die Klasse implementiert die Schnittstelle Struktur, d.h. auf bestimmte
	 * Attribute kann mit dem Elementzugriffoperator zugegriffen werden.
	 * Folgende Attribute werden unterstützt:
	 * <ul>
	 * <li><b>aktiv</b>: liefert den Zustand der Aktivität als Wahrheitswert
	 * </li>
	 * <li><b>abgebrochen</b>: liefert den Fehlerzustand der Aktivität als
	 * Wahrheitswert</li>
	 * <li><b>fehler</b>: liefert das Fehlerobjekt der Aktivität, dass beim
	 * Abbruch gesetzt wird oder den Wert "undefiniert"</li>
	 * </ul>
	 *
	 * @see #setStrukturElement(String, Object)
	 */
	@Override
	public Object getStrukturElement(final String name) {
		Object ergebnis = null;
		if ("aktiv".equals(name)) {
			ergebnis = LogischerWert.of(isRunning());
		} else if ("abgebrochen".equals(name)) {
			ergebnis = LogischerWert.of(fehler != null);
		} else if ("fehler".equals(name)) {
			if (fehler != null) {
				ergebnis = fehler;
			} else {
				ergebnis = NichtWert.NICHTWERT;
			}
		} else {
			throw new ObjektZugriffsFehler(UdaFehlerSubtyp.ELEMENT);
		}
		return ergebnis;
	}

	/**
	 * {@inheritDoc}.<br>
	 * Die Attribute einer Aktivität können von aussen nicht gesetzt werden. Die
	 * Funktion verursacht in jedem Fall einen ObjektzugriffsFehler.
	 *
	 * @see #getStrukturElement(String)
	 */
	@Override
	public void setStrukturElement(final String name, final Object daten) {
		throw new ObjektZugriffsFehler(UdaFehlerSubtyp.ARGUMENTE);
	}

	/**
	 * liefert den lokalen Kontext der Anweisung.
	 *
	 * @return kontext den Kontext
	 */
	protected SkriptKontext getKontext() {
		return kontext;
	}

	/**
	 * setzt den Fehler, der zum Abbruch der Aktivität geführt hat.
	 *
	 * @param fehler
	 *            festzulegender Fehler
	 */
	protected void setFehler(final UdaFehler fehler) {
		this.fehler = fehler;
	}
}
