/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.container;

import de.bsvrz.ibv.uda.interpreter.daten.fehler.ContainerZugriffsFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehlerSubtyp;
import de.bsvrz.sys.funclib.bitctrl.interpreter.logik.LogischerWert;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Die Klasse beschreibt einen Iterator, über dem über die Elemente eines
 * UdaContainers operiert werden kann. Im Gegensatz zu den Standarditeratoren in
 * der Java-Implementierung sind UdaIteratoren mit den Containern fest
 * verbunden, um die Ãnderungen des Containerinhalts über einen auf diesen
 * gesetzten Iterator oder einen Container zwischen allen Iteratoren abgeglichen
 * wird.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class UdaIterator {

	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * Konstante zur Definition der Anfangsposition (vor dem ersten Element).
	 */
	public static final int ANFANG = 0;

	/**
	 * Konstante zur Definition der Endposition (nach dem letzten Element).
	 */
	public static final int ENDE = Integer.MAX_VALUE;

	/**
	 * der Container, für den der Iterator angelegt wurde.
	 */
	private final UdaContainer container;

	/**
	 * die aktuelle Iteratorposition. Das erste Element innerhalb eines
	 * Containers hat die Position 1.
	 */
	private int position;

	/**
	 * das Element auf das der Iterator verweist.
	 */
	private Object element;

	/**
	 * Markierung, ob der Iterator auf ein gültiges Element verweist.
	 */
	private boolean valid;

	/**
	 * der Schlüssel des Elements, auf das der Iterator im Moment verweist.
	 */
	private String schluessel;

	/**
	 * Konstruktor erzeugt einen Iterator, der über dem angegebenen Container
	 * operiert.<br>
	 * Die Position wird auf den übergebenen Initialwert gesetzt. Ein Index von
	 * 0 positioniert den Iterator vor das erste Element. Ein Index, der größer
	 * als die Anzahl Elemente ist positioniert den Iterator hinter das letzte
	 * Element des Containers. Ein Index der kleiner als die negative Anzahl der
	 * Elemente ist positioniert den Iterator vor das erste Element.
	 *
	 * @param container
	 *            der Container mit dem der Iterator verbunden ist
	 * @param initialePosition
	 *            die initialie Position
	 */
	UdaIterator(final UdaContainer container, final Number initialePosition) {
		this.container = container;
		this.position = initialePosition.intValue();
		if (position < 0) {
			position = container.getGroesse() + 1 + position;
		}
		if (position <= 0) {
			position = UdaIterator.ANFANG;
		} else if (position > container.getGroesse()) {
			position = UdaIterator.ENDE;
		} else {
			try {
				this.element = container.getContainerElement(position);
				this.schluessel = container.getSchluessel(position);
			} catch (final ContainerZugriffsFehler e) {
				UdaIterator.LOGGER.warning(e.getLocalizedMessage());
				// element bleibt auf dem Wert null
			}
			if (element != null) {
				valid = true;
			}
		}
	}

	/**
	 * liefert eine Repräsentation des Iterators als Zeichenkette.<br>
	 * Die Ausgabe erfolgt in der Form "[&lt;element&gt;]". Ist der Iterator vor
	 * oder hinter die Elemente des Containers positioniert, wird an Stelle des
	 * Elementses die Zeichenkette "ANFANG" bzw. "ENDE" eingesetzt.
	 * {@inheritDoc}.
	 *
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		final StringBuffer buffer = new StringBuffer("[");
		if (position == UdaIterator.ANFANG) {
			buffer.append("ANFANG");
		} else if (position == UdaIterator.ENDE) {
			buffer.append("ENDE");
		} else {
			buffer.append(getElement());
		}
		buffer.append(']');
		return buffer.toString();
	}

	/**
	 * liefert den Container, dem der Iterator zugeordnet ist.
	 *
	 * @return der Container
	 */
	public UdaContainer getContainer() {
		return container;
	}

	/**
	 * liefert das Element, auf das der Iterator verweist.<br>
	 * Ein Element wird auch geliefert, wenn es über den Iterator aus dem
	 * Container entfernt wurde und der Iterator nocht nicht bewegt worden
	 * ist.Wenn der Container vor das erste Element oder hinter das letzte
	 * Element positioniert ist, dann liefert die Methode einen
	 * <i>ContainerZugriffsFehler</i> zurück.
	 *
	 * @return element das Element auf das der Iterator verweist
	 * @see #setElement(Object)
	 */
	public Object getElement() {
		if (innerhalb().get()) {
			element = container.getContainerElement(position);
			schluessel = container.getSchluessel(position);
		}
		if (element == null) {
			throw new ContainerZugriffsFehler(UdaFehlerSubtyp.SELEKTION,
					"Iterator verweist nicht auf ein gültiges Element");
		}
		return element;
	}

	/**
	 * die Funktion ersetzt das Element, auf das der Iterator innerhalb des
	 * zugeordneten Containers verweist, durch das übergebene Objekt.<br>
	 * Wenn der Iterator vor das erste Element oder hinter das letzte Element
	 * innerhalb des Containers positioniert ist, dann liefert die Methode einen
	 * ContainerZugriffsFehler zurück. Ein ContainerZugriffsFehler entsteht
	 * ebenfalls, wenn das Element auf das der Iterator verweist über diesen aus
	 * dem Container entfernt wurde.
	 *
	 * @param daten
	 *            das neue Datenobjekt
	 * @see #getElement()
	 */
	public void setElement(final Object daten) {
		if (innerhalb().get()) {
			container.setContainerElement(position, daten);
		} else {
			throw new ContainerZugriffsFehler(UdaFehlerSubtyp.SELEKTION,
					"Iterator verweist nicht auf ein gültiges Element innerhalb des Containers");
		}
	}

	/**
	 * Die Funktion aktualisiert den Iterator, wenn in der verbundenen Liste ein
	 * Element hinzugefügt wurde.
	 *
	 * @param index
	 *            die Position, an der ein Element hinzugefügt wurde.
	 */
	void elementHinzugefuegt(final int index) {
		if ((position != UdaIterator.ENDE) && (position >= index)) {
			position++;
		}
	}

	/**
	 * Die Funktion aktualisiert den Iterator, wenn in der verbundenen Liste ein
	 * Element entfernt wurde.
	 *
	 * @param index
	 *            die Position, an der ein Element entfernt wurde.
	 */
	public void elementEntfernt(final int index) {
		if (position == index) {
			valid = false;
		} else if (index < position) {
			if (position != UdaIterator.ENDE) {
				position--;
			}
		}
	}

	/**
	 * Die Funktion ermittelt, ob der Iterator auf ein gültiges ELement
	 * verweist, d.h. ein Element das sich innerhalb des zugeordneten Containers
	 * befindet.
	 *
	 * @return valid "wahr", wenn auf ein gültiges Element verwiesen wird.
	 */
	public LogischerWert innerhalb() {
		boolean result = valid;
		if ((position == UdaIterator.ANFANG) || (position == UdaIterator.ENDE)) {
			result = false;
		}
		return LogischerWert.of(result);
	}

	/**
	 * Die Funktion erzeugt einen Iterator.<br>
	 * Der neue Iterator ist mit dem gleichen Container verbunden und ist
	 * initial auf das Vorgängerelement gegenüber dem Element auf das der
	 * Iterator verweist positioniert.
	 *
	 * @return der neu angelegte Iterator
	 */
	public UdaIterator getVorgaengerIterator() {
		final int neuePosition = prevIndex();
		return container.getUdaIterator(neuePosition);
	}

	/**
	 * die Funktion positioniert den Iterator auf das vorhergehende Element.<br>
	 * Das Ergebnis ist der logische Wert "wahr", wenn die Positionierung
	 * ausgeführt werden konnte.
	 *
	 * @return das Ergebnis der Positionierung
	 */
	public LogischerWert zurueck() {
		position = prevIndex();
		if (position == UdaIterator.ANFANG) {
			element = null;
			valid = false;
		} else {
			valid = false;
			try {
				element = container.getContainerElement(position);
				schluessel = container.getSchluessel(position);
				valid = true;
			} catch (final ContainerZugriffsFehler e) {
				UdaIterator.LOGGER.warning(e.getLocalizedMessage());
				element = null;
			}
		}
		return innerhalb();
	}

	/**
	 * die Funktion positioniert den Iterator auf das nachfolgende Element.<br>
	 * Das Ergebnis ist der logische Wert "wahr", wenn die Positionierung
	 * ausgeführt werden konnte.
	 *
	 * @return das Ergebnis der Positionierung
	 */
	public LogischerWert weiter() {
		position = nextIndex();
		if (position == UdaIterator.ENDE) {
			element = null;
			valid = false;
		} else {
			valid = false;
			try {
				element = container.getContainerElement(position);
				schluessel = container.getSchluessel(position);
				valid = true;
			} catch (final ContainerZugriffsFehler e) {
				UdaIterator.LOGGER.warning(e.getLocalizedMessage());
				element = null;
			}
		}
		return innerhalb();
	}

	/**
	 * Die Funktion ermittelt den Index des vorhergehenden Elements.
	 *
	 * @return den Indexwert
	 */
	private int prevIndex() {
		int neuePosition = UdaIterator.ANFANG;
		if (position == UdaIterator.ANFANG) {
			neuePosition = UdaIterator.ANFANG;
		} else if (position == UdaIterator.ENDE) {
			neuePosition = container.getGroesse();
		} else {
			neuePosition = position - 1;
		}
		if (neuePosition < 0) {
			neuePosition = UdaIterator.ANFANG;
		}
		return neuePosition;
	}

	/**
	 * Die Funktion erzeugt einen Iterator.<br>
	 * Der neue Iterator ist mit dem gleichen Container verbunden und ist
	 * initial auf das Nachfolgerelement gegenüber dem Element auf das der
	 * Iterator verweist positioniert.
	 *
	 * @return der neu angelegte Iterator
	 */
	public Object getNachfolgerIterator() {
		final int neuePosition = nextIndex();
		return container.getUdaIterator(neuePosition);
	}

	/**
	 * Die Funktion ermittelt den Index des nächstfolgenden Elements.
	 *
	 * @return den Indexwert
	 */
	private int nextIndex() {
		int neuePosition = UdaIterator.ENDE;
		if (position == UdaIterator.ANFANG) {
			neuePosition = 1;
		} else if (position == UdaIterator.ENDE) {
			neuePosition = UdaIterator.ENDE;
		} else {
			neuePosition = position + 1;
		}
		if (neuePosition > container.getGroesse()) {
			neuePosition = UdaIterator.ENDE;
		}
		return neuePosition;
	}

	/**
	 * liefert die aktuelle Position des Iterators.
	 *
	 * @return position
	 */
	public int getPosition() {
		return position;
	}

	/**
	 * liefert den Schlüssel des Elements, auf das der Iterator verweist.
	 *
	 * @return den Schluessel
	 */
	public Object schluessel() {
		Object result = schluessel;
		if (schluessel == null) {
			result = new ContainerZugriffsFehler(UdaFehlerSubtyp.INDEX);
		}

		return result;
	}
}
