/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.ausdruck;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.ibv.uda.interpreter.AnweisungsBlock;
import de.bsvrz.ibv.uda.interpreter.anweisung.RueckgabeAnweisung;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.Fehler;
import de.bsvrz.ibv.uda.interpreter.daten.konstante.ZeichenkettenLiteral;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext;
import de.bsvrz.sys.funclib.bitctrl.interpreter.logik.LogischerWert;

/**
 * ein Ausdruck zur Darstellung eines nutzerdefinierten Quantors.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class QuantorDefinition implements Ausdruck {
	/**
	 * der Name des Quantors.
	 */
	private final String name;

	/**
	 * der Name der Menge, die an den Quantor übergeben wird.
	 */
	private final String mengenName;

	/**
	 * der Name der Bedingung, die als Funktion übergeben wird.
	 */
	private final String bedingungsName;

	/**
	 * der Anweisungsblock, der die Anweisung zur ermittlung des Wertes des
	 * Quantors enthalten.
	 */
	private final AnweisungsBlock anweisungen;

	/**
	 * Konstruktor.<br>
	 *
	 * @param name
	 *            der Name des nutzerdefinierten Quantors
	 * @param mengenName
	 *            der Name der Menge über die eine Auswertung erfolgen soll
	 * @param bedingungsName
	 *            der Name der Bedingung mit der eine Auswertung erfolgen soll
	 * @param anweisungen
	 *            die Anweisungen, die die Funktionalität des Quantors
	 *            beschreiben
	 */
	public QuantorDefinition(final String name, final String mengenName, final String bedingungsName,
			final AnweisungsBlock anweisungen) {
		this.name = name;
		this.mengenName = mengenName;
		this.anweisungen = anweisungen;
		this.bedingungsName = bedingungsName;
	}

	/**
	 * liefert den Name für die Bedingungsfunktion.
	 *
	 * @return den Name
	 */
	String getBedingungsName() {
		return bedingungsName;
	}

	/**
	 * liefert den Namen der Menge.
	 *
	 * @return den Namen
	 */
	protected String getMengenName() {
		return mengenName;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#getNachfolger()
	 */
	@Override
	public List<Ausdruck> getNachfolger() {
		final List<Ausdruck> liste = new ArrayList<>();
		liste.add(new ZeichenkettenLiteral(name));
		liste.add(new ZeichenkettenLiteral(mengenName));
		liste.add(anweisungen);
		return liste;
	}

	/**
	 * liefert den Name der Quantordefinition.
	 *
	 * @return den Name
	 */
	public String getName() {
		return name;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#interpret(de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext)
	 */
	@Override
	public Object interpret(final Kontext kontext) {
		Object result = anweisungen.interpret(kontext);
		if (result instanceof RueckgabeAnweisung) {
			result = ((RueckgabeAnweisung) result).getRueckgabeWert();
		}
		if (!(result instanceof LogischerWert)) {
			throw new Fehler("Quantor \"" + name
					+ "\" liefert keinen logischen Wert");
		}
		return result;
	}
}
