/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.dav;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.Attribute;
import de.bsvrz.dav.daf.main.config.AttributeListDefinition;
import de.bsvrz.dav.daf.main.config.AttributeType;
import de.bsvrz.dav.daf.main.config.DoubleAttributeType;
import de.bsvrz.dav.daf.main.config.IntegerAttributeType;
import de.bsvrz.dav.daf.main.config.IntegerValueRange;
import de.bsvrz.ibv.uda.interpreter.daten.container.Feld;
import de.bsvrz.sys.funclib.bitctrl.interpreter.NichtWert;
import de.bsvrz.sys.funclib.bitctrl.interpreter.logik.LogischerWert;

/**
 * die Repräsentation eines Datenverteiler-<i>AttributListDefinition</i>-Objekts
 * in einem Uda-Skript.<br>
 * Das Objekt hat, bis auf die vom Typ <i>KonfigurationsObjekt</i> vererbten:
 * <ul>
 * <li>pid</li>
 * <li>id</li>
 * <li>name</li>
 * <li>info</li>
 * <li>typ</li>
 * <li>gültig</li>
 * <li>attributgruppen</li>
 * <li>mengen</li>
 * </ul>
 * folgende zusätzliche Attribute:
 * <ul>
 * <li><b>typ:</b> liefert den Typ des Attributs</li>
 * <li><b>einheit:</b> liefert die Einheit, wenn es sich um numerisches Attribut
 * handelt</li>
 * <li><b>immerDefiniert:</b> ermittelt, ob ein Standardwert für das Attribut
 * definiert ist.</li>
 * <li><b>wertebereich:</b> liefert den Wertebereich (bei Integerattributen)
 * </li>
 * <li><b>attribute:</b> liefert die Attribute, wenn es sich um eine
 * Attributliste handelt</li>
 * </ul>
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class AttributObjekt extends KonfigurationsObjekt {

	/**
	 * Konstruktor.
	 *
	 * @param verbindung
	 *            die verwendete Datenverteilerverbindung
	 * @param object
	 *            das repräsentierte Datenverteilerobjekt
	 */
	public AttributObjekt(final ClientDavInterface verbindung, final Attribute object) {
		super(verbindung, object);
	}

	/**
	 * {@inheritDoc}.<br>
	 * Die Funktion liefert immer ein <i>Attribute</i>-Objekt im Gegensatz zur
	 * allgemeineren Definition in <i>DavDaten</i>.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.daten.dav.DavDaten#getObjekt()
	 */
	@Override
	public Attribute getObjekt() {
		return (Attribute) super.getObjekt();
	}

	/**
	 * {@inheritDoc}.<br>
	 * Die Funktion erlaubt den Zugriff auf folgende Attribute:
	 * <ul>
	 * <li>pid: die PID des Attributs</li>
	 * <li>id: die ID des Attributs</li>
	 * <li>name: der Name des Attributs</li>
	 * <li>info: das Info-Objekt des Attributs (enthält <i>kurzinfo</i> und
	 * <i>beschreibung</i></li>
	 * <li>typ: der Typ des Attributs</li>
	 * <li><b>einheit:</b> liefert die Einheit, wenn es sich um numerisches
	 * Attribut handelt</li>
	 * <li><b>immerDefiniert:</b> ermittelt, ob ein Standardwert für das
	 * Attribut definiert ist.</li>
	 * <li><b>wertebereich:</b> liefert den Wertebereich (bei Integerattributen)
	 * </li>
	 * <li><b>attribute:</b> liefert die Attribute, wenn es sich um eine
	 * Attributliste handelt</li>
	 * </ul>
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.daten.dav.KonfigurationsObjekt#getStrukturElement(java.lang.String)
	 */
	@Override
	public Object getStrukturElement(final String name) {
		Object result = null;

		final Attribute object = getObjekt();

		if ("typ".equals(name)) {
			result = DavDaten.erzeugeObjekt(getVerbindung(), object.getAttributeType());
		} else if ("einheit".equals(name)) {
			result = "";
			final AttributeType type = object.getAttributeType();
			if (type instanceof DoubleAttributeType) {
				result = ((DoubleAttributeType) type).getUnit();
			} else if (type instanceof IntegerAttributeType) {
				final IntegerValueRange range = ((IntegerAttributeType) type).getRange();
				if (range != null) {
					result = range.getUnit();
				}
			}
		} else if ("immerDefiniert".equals(name)) {
			result = LogischerWert.of(object.getDefaultAttributeValue() != null);
		} else if ("istEinFeld".equals(name)) {
			result = LogischerWert.of(object.isArray());
		} else if ("wertebereich".equals(name)) {
			final AttributeType type = object.getAttributeType();
			if (type instanceof IntegerAttributeType) {
				final IntegerValueRange range = ((IntegerAttributeType) type).getRange();
				if (range != null) {
					result = DavDaten.erzeugeObjekt(getVerbindung(), range);
				}
			}
			if (result == null) {
				result = NichtWert.NICHTWERT;
			}
		} else if ("attribute".equals(name)) {
			final AttributeType type = object.getAttributeType();
			if (type instanceof AttributeListDefinition) {
				final AttributeListDefinition definition = (AttributeListDefinition) type;
				result = new Feld();
				for (final Attribute att : definition.getAttributes()) {
					((Feld) result).einfuegen(0, DavDaten.erzeugeObjekt(getVerbindung(), att));
				}
			} else {
				result = NichtWert.NICHTWERT;
			}
		}

		if (result == null) {
			result = super.getStrukturElement(name);
		}

		return result;
	}
}
