/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.dav;

import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Repräsentation der Daten einer Datenverteiler-Sendeanmeldung.<br>
 * Instanzen dieser Klasse dienen lediglich der Verwaltung der Anmeldedaten. Die
 * eigentliche Anmeldung beim Datenverteiler wird nicht ausgeführt.
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public class SenderAnmeldung implements Comparable<SenderAnmeldung> {

	/** das Systemobjekt, für das die Anmeldung erfolgt ist. */
	private final SystemObject objekt;

	/** die Datenbeschreibung, für die die Anmeldung erfolgt. */
	private final DataDescription desc;

	/**
	 * markiert, ob eine Quellenanmeldung die Datenbeschreibung, für die die
	 * Anmeldung erfolgt.
	 */
	private final boolean quelle;

	/** der Status der Sendesteuerung. */
	private byte status = ClientSenderInterface.STOP_SENDING;

	/**
	 * die Anzahl der Anmeldungen.
	 */
	private int anzahl;

	/**
	 * Konstruktor.<br>
	 * Erzeugt die Datenrepräsentation einer Sendeanmeldung für die gegebene
	 * Kombination aus Systemobjekt und Datenbeschreibung.
	 *
	 * @param objekt
	 *            das Systemobjekt
	 * @param desc
	 *            die Datenbeschreibung
	 * @param quelle
	 *            Anmeldung als Quelle oder Sender
	 */
	public SenderAnmeldung(final SystemObject objekt, final DataDescription desc, final boolean quelle) {
		this.objekt = objekt;
		this.desc = desc;
		this.quelle = quelle;
		anzahl = 1;
	}

	/**
	 * inkrementiert die Anzahl der Anmeldung für die entsprechende Kombination
	 * aus Systemobjekt und Datenbeschreibung.
	 */
	public void add() {
		anzahl++;
	}

	/**
	 * vergleicht zwei Sendeanmeldungen miteinander.<br>
	 * Die Funktion implementiert die entsprechende Funktion der Schnittstelle
	 * Comparable. {@inheritDoc}.
	 *
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	@Override
	@SuppressWarnings("unchecked")
	public int compareTo(final SenderAnmeldung anmeldung) {
		int result = 0;
		result = objekt.compareTo(anmeldung.objekt);
		if (result == 0) {
			result = desc.getAttributeGroup().compareTo(anmeldung.desc.getAttributeGroup());
			if (result == 0) {
				result = desc.getAspect().compareTo(anmeldung.desc.getAspect());
			}
		}
		return result;
	}

	/** {@inheritDoc} */
	@Override
	public boolean equals(final Object obj) {
		boolean result = false;

		if ((obj != null) && (getClass() == obj.getClass())) {
			if (this == obj) {
				result = true;
			} else {
				final SenderAnmeldung other = (SenderAnmeldung) obj;
				if (objekt.equals(other.objekt)) {
					if (desc.getAttributeGroup().equals(other.desc.getAttributeGroup())) {
						if (desc.getAspect().equals(other.desc.getAspect())) {
							result = true;
						}
					}
				}
			}
		}
		return result;
	}

	/**
	 * liefert den Status der Sendesteuerung für die Anmeldung.
	 *
	 * @return den Status
	 */
	public byte getStatus() {
		return status;
	}

	/** {@inheritDoc} */
	@Override
	public int hashCode() {
		return objekt.hashCode();
	}

	/**
	 * ermittelt, ob eine Quellenanmeldung vorliegt.
	 *
	 * @return <code>true</code>, wenn eine Anmeldung als Quelle erfolgte
	 */
	public boolean isQuelle() {
		return quelle;
	}

	/**
	 * dekrementiert die Anzahl der Anmeldung für die entsprechende Kombination
	 * aus Systemobjekt und Datenbeschreibung.
	 */
	public void remove() {
		anzahl--;

	}

	/**
	 * setzt den Status der Sendesteuerung der Anmeldung.
	 *
	 * @param status
	 *            der Status
	 */
	public void setStatus(final byte status) {
		this.status = status;
	}

	/**
	 * liefert die Anzahl der registrierten Anmeldungen für die entsprechende
	 * Kombination aus Systemobjekt und Datenbeschreibung.
	 *
	 * @return die Anzahl
	 */
	public int size() {
		return anzahl;
	}
}
