/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.verwaltung.protokoll;

import java.io.*;
import java.nio.charset.StandardCharsets;

import com.bitctrl.Constants;

import de.bsvrz.ibv.uda.uda.data.SkriptLauf;
import de.bsvrz.ibv.uda.verwaltung.ServerSkriptLauf;
import de.bsvrz.sys.funclib.operatingMessage.MessageGrade;
import de.bsvrz.sys.funclib.operatingMessage.MessageSender;
import de.bsvrz.sys.funclib.operatingMessage.MessageType;

/**
 * beschreibt die Protokollierung der Ausgabedaten eines Uda-Skripts in eine
 * Datei.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class DateiProtokoll extends Protokoll {

	/**
	 * Versions-ID für die Serialisierung des Objekts.
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * der Writer zur Ausgabe der Skriptdaten.
	 */
	private Writer writer;

	/**
	 * Standardkonstruktor, erzeugt eine Instanz des Protokolltyps.
	 */
	public DateiProtokoll() {
		super(ProtokollTyp.DATEI);
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.verwaltung.protokoll.Protokoll#abschliessen()
	 */
	@Override
	public void abschliessen() {
		if (writer != null) {
			try {
				writer.close();
			} catch (final IOException e) {
				// schliessen nicht erfolgreich, wird ignoriert, es wird
				// lediglich eine Betriebsmeldung versendet
				MessageSender.getInstance().sendMessage(MessageType.APPLICATION_DOMAIN, MessageGrade.ERROR,
						e.getMessage());
			}
			writer = null;
		}
	}

	/**
	 * {@inheritDoc}.<br>
	 * Die Funktion gibt die vom Skriptlauf übergebenen Texte in die zuvor
	 * angelegte und geöffnete Datei aus.<br>
	 * Wenn keine Datei angelegt wurde, wird die Ausgabeanweisung ignoriert.
	 * IO-Fehler bei der Ausgabe werden ebenfalls ignoriert, d.h. fehlende
	 * Schreibberechtigung, volle Festplatte usw. führen nicht zu einem Fehler
	 * innerhalb der Anwendung. Es wird lediglich eine Fehlermeldung an die
	 * Betriebsmeldungsverwaltung versendet und der Ausgabekanal wird
	 * geschlossen.
	 *
	 * @see de.bsvrz.ibv.uda.verwaltung.protokoll.Protokoll#ausgeben(de.bsvrz.ibv.uda.verwaltung.protokoll.ProtokollEintrag)
	 */
	@Override
	public void ausgeben(final ProtokollEintrag daten) {
		if (writer != null) {
			try {
				writer.write(daten.getText());
				writer.write(Constants.NL);
			} catch (final IOException e) {
				// Meldung wird versendet, der Ausgabekanal wird geschlossen, um
				// eine wiederholte Ausgabe der Betriebsmeldung zu verhindern
				MessageSender.getInstance().sendMessage(MessageType.APPLICATION_DOMAIN, MessageGrade.ERROR,
						e.getMessage());
				abschliessen();
			}
		}
	}

	/**
	 * {@inheritDoc}.<br>
	 * Die Funktion legt die Ausgabedatei für den übergebenen Skriptlauf an.<br>
	 * Handelt es sich bei dem übergebenen Skriptlauf um ein Serverobjekt, d.h.
	 * ein Objekt innerhalb der "Umfassenden Datenanalyse", wird die Datei
	 * innerhalb des Verzeichnisses &lt;UdaModul-PID&gt;/protokoll abgelegt.
	 * Wurde kein Dateiname festgelegt, ergibt sich der Name als
	 * <i>&lt;Skriptlauf-PID&gt;.prot</i><br>
	 * Bei allen anderen Skriptluaftypen wird der innerhalb der Protokollklasse
	 * definierte Name als Dateiname angenommen.
	 *
	 * @see de.bsvrz.ibv.uda.verwaltung.protokoll.Protokoll#initialisiere(de.bsvrz.ibv.uda.uda.data.SkriptLauf)
	 */
	@Override
	public void initialisiere(final SkriptLauf lauf) {
		writer = null;
		String name = getName();
		if (lauf instanceof ServerSkriptLauf) {
			final ServerSkriptLauf serverLauf = (ServerSkriptLauf) lauf;

			final File defaultDir = new File(serverLauf.getModul().getModulVerzeichnis(), "protokoll");
			defaultDir.mkdirs();

			if ((name == null) || (name.length() <= 0)) {
				name = serverLauf.getObjekt().getPid() + ".prot";
			}

			File file = new File(name);
			if (!file.isAbsolute()) {
				file = new File(defaultDir, name);
			}

			try {
				writer = new OutputStreamWriter(new FileOutputStream(file.getAbsolutePath()), StandardCharsets.UTF_8);
			} catch (final IOException e) {
				e.printStackTrace();
				writer = null;
			}
		} else {
			if (name != null) {
				try {
					writer = new OutputStreamWriter(new FileOutputStream(new File(name)), StandardCharsets.UTF_8);
				} catch (final IOException e) {
					e.printStackTrace();
					writer = null;
				}
			}
		}
	}
}
